package LATMOS::Accounts::Web::Controller::Summary::ListEmployments;
use Moose;
use namespace::autoclean;
use Date::Parse;
use DateTime;

BEGIN { extends 'Catalyst::Controller'; }

=head1 NAME

LATMOS::Accounts::Web::Controller::Summary::ListEmployments - Catalyst Controller

=head1 DESCRIPTION

Catalyst Controller.

=head1 METHODS

=cut


=head2 index

=cut

sub index :Path :Args(0) {
    my ( $self, $c ) = @_;

    my $base = $c->model('Accounts')->db;

    my ($min, $max) = $base->getEmploymentRange;
    $c->stash->{minE} = $min
        ? DateTime->from_epoch( epoch => str2time( $min ) )
        : DateTime->now->subtract( days => 1 );
    $c->stash->{maxE} = $max
        ? DateTime->from_epoch( epoch => str2time( $max ) )
        : DateTime->now->add( days => 1 );

    $c->stash->{min} = $c->req->param('min')
        ? DateTime->from_epoch( epoch => str2time( $c->req->param('min') ) )
        : DateTime->now->subtract( days => 1 );
    $c->stash->{max} = $c->req->param('max')
        ? DateTime->from_epoch( epoch => str2time( $c->req->param('max') ) )
        : DateTime->now->subtract( days => 1 );


    my @search = (
        'firstday<=' . $c->stash->{max}->ymd('-'),
        'lastday>='  . $c->stash->{min}->ymd('-'),
        'lastday=NULL',
    );

    my %atstart = map { $_ => 1 } $base->search_objects(
        'employment', sprintf('firstday>=%s', $c->stash->{min}->ymd('-'))
    );
    my %atend = map { $_ => 1 } $base->search_objects(
        'employment', sprintf('lastday<=%s', $c->stash->{max}->ymd('-'))
    );

    if (my $jtype = $c->req->param('jtype')) {
        push(@search, 'contratType.sutype=' . $jtype);
    }
    foreach ($c->req->param('contratType')) {
        push(@search, 'contratType=' . $_);
    }
    
    my %users;

    foreach my $attr (qw(sn givenName description)) {
        my %res = $base->attributes_summary_by_object('user', $attr);
        $users{$_}{$attr} = $res{$_}->[0] foreach (keys %res);
    }
    foreach (keys %users) {
        $c->stash->{users}{$_} = $users{$_}{sn} && $users{$_}{givenName}
            ? "$users{$_}{givenName} $users{$_}{sn}"
            : $users{$_}{description}
    }

    my %inclusiveObjects = map { $_ => {} } $base->search_objects('employment', @search);
    my %objects = ();
    my $atstart = $c->req->param('atstart');
    my $atend =   $c->req->param('atend');

    foreach my $emp (keys %inclusiveObjects) {
        if ($atstart) {
            $atstart{$emp} or next;
        }
        if ($atend) {
            $atend{$emp} or next;
        }
        $objects{$emp} = $inclusiveObjects{$emp};
    }

    my @attrs = qw(user firstday lastday contratType department);

    foreach my $attr (@attrs) {
        my %res = $c->model('Accounts')->db->attributes_summary_by_object('employment', $attr);
        foreach my $obj (keys %res) {
            exists($objects{$obj}) or next;
            $objects{$obj}{$attr} = $res{$obj}->[0];
        }
    }

    my @array = ();

    if ($c->req->param('merge')) {
        my $employmentDelay = ( $base->config('employment_delay') || 0 ) + 1;

        foreach (sort { $a->{user} cmp $b->{user} || $a->{firstday} cmp $b->{firstday} } values %objects) {
            if (my $prev = $array[-1]) {
                if ($prev->{user} eq $_->{user}
                    && $prev->{contratType} eq $_->{contratType}) {

                    my $pEnd = DateTime->from_epoch(epoch => str2time($prev->{lastday}));

                    $pEnd->add(days => $employmentDelay);
                    if ($pEnd->ymd('-') ge $_->{firstday}) {
                        $prev->{lastday} = $_->{lastday};

                        if (my $dpmt = $_->{department}) {
                            if (!$prev->{department}) {
                                $prev->{department} = $dpmt;
                            } elsif ($prev->{department} !~ m/\b$dpmt\b/) {
                                $prev->{department} .= ", $dpmt";
                            }
                        }
                        next;
                    }
                }
            }

            push(@array, $_);
        }
    } else {
        @array = values %objects;
    }

    foreach (keys %objects) {
        $objects{$_}{username} = $c->stash->{users}{ $objects{$_}{user} };
    }

    $c->stash->{employments} = [ sort { $a->{user} cmp $b->{user} || $b->{firstday} cmp $a->{firstday} } @array ];
}



=encoding utf8

=head1 AUTHOR

Olivier Thauvin,Guyancourt - B1428,+33 1 80285052,

=head1 LICENSE

This library is free software. You can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

__PACKAGE__->meta->make_immutable;

1;
